/* Invacto Fluo */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.Turbolinks = {}));
}(this, (function (exports) { 'use strict';

  function tokenize(query) {
      return query.trim().toLowerCase().split(' ');
  }
  var tokenizer = {
      tokenize: tokenize,
  };

  function findPositionOftoken(text, token, startAt) {
      var index = text.indexOf(token, startAt);
      if (index < 0)
          return null;
      return { start: index, length: token.length };
  }
  function findPositionsOfToken(text, token) {
      if (token.length === 0)
          return [];
      var start = 0;
      var results = [];
      var match = findPositionOftoken(text, token, start);
      while (match != null) {
          results.push({ start: match.start, length: token.length });
          start = match.start + token.length;
          match = findPositionOftoken(text, token, start);
      }
      return results;
  }
  function findMatches(text, query) {
      var tokens = tokenizer.tokenize(query);
      return findPositionsOfAllTokens(text, tokens);
  }
  function findPositionsOfAllTokens(text, tokens) {
      var lowerCaseText = text.toLowerCase();
      return tokens.reduce(function (result, token) {
          var positionsOfCurrentToken = findPositionsOfToken(lowerCaseText, token);
          result.push.apply(result, positionsOfCurrentToken);
          return result;
      }, []);
  }
  function createHighlightListFromMatches(text, matches) {
      var highlightList = [];
      var start = 0;
      matches.forEach(function (match) {
          var prefix = text.slice(start, match.start);
          if (prefix.length > 0) {
              highlightList.push({ value: prefix, highlighted: false });
          }
          var matchedText = text.slice(match.start, match.start + match.length);
          highlightList.push({ value: matchedText, highlighted: true });
          start = match.start + match.length;
      });
      var remainder = text.slice(start);
      if (remainder.length > 0) {
          highlightList.push({ value: remainder, highlighted: false });
      }
      return highlightList;
  }
  function removeOverlappingMatches(matches) {
      var resultMatches = [];
      for (var i = 0; i < matches.length; i++) {
          var currentMatch = matches[i];
          var nextMatch = matches[i + 1];
          while (nextMatch != null && currentMatch.length + currentMatch.start >= nextMatch.start) {
              currentMatch.length = (nextMatch.length + nextMatch.start) - currentMatch.start;
              nextMatch = matches[++i + 1];
          }
          resultMatches.push(currentMatch);
      }
      return resultMatches;
  }
  function createHighlightList(text, query) {
      var tokens = tokenizer.tokenize(query);
      return createHighlightListFromTokens(text, tokens);
  }
  function createHighlightListFromTokens(text, tokens) {
      var matches = findPositionsOfAllTokens(text, tokens);
      var sortedMatches = matches.sort(function (a, b) { return a.start - b.start; });
      var uniqueMatches = removeOverlappingMatches(sortedMatches);
      var res = createHighlightListFromMatches(text, uniqueMatches);
      return res;
  }
  function stringContainsAnyOfQuery(text, query) {
      var tokens = tokenizer.tokenize(query);
      return stringContainsAnyTokens(text, tokens);
  }
  function stringContainsAnyTokens(text, tokens) {
      var lowerCaseText = text.toLowerCase();
      return tokens.some(function (token) { return lowerCaseText.includes(token); });
  }
  var highlighter = {
      createHighlightList: createHighlightList,
      createHighlightListFromTokens: createHighlightListFromTokens,
      stringContainsAnyOfQuery: stringContainsAnyOfQuery,
      stringContainsAnyTokens: stringContainsAnyTokens,
      findMatches: findMatches
  };

  function highlightHTMLElement(element, query) {
      var tokens = tokenizer.tokenize(query);
      return highlightHTMLElementWithTokens(element, tokens);
  }
  function highlightHTMLElementWithTokens(element, tokens) {
      var children = element.childNodes;
      children.forEach(function (child) {
          if (child.nodeType === Node.ELEMENT_NODE) {
              highlightHTMLElementWithTokens(child, tokens);
          }
          if (child.nodeType === Node.TEXT_NODE) {
              highlightTextNode(child, tokens);
          }
      });
  }
  function highlightTextNode(textNode, tokens) {
      var text = textNode.data;
      var highlightList = highlighter.createHighlightListFromTokens(text, tokens);
      if (highlightList.length === 1)
          return;
      var parent = textNode.parentElement;
      for (var _i = 0, highlightList_1 = highlightList; _i < highlightList_1.length; _i++) {
          var highlightItem = highlightList_1[_i];
          if (highlightItem.highlighted) {
              var markedNode = createMarkedNode(text);
              parent === null || parent === void 0 ? void 0 : parent.insertBefore(textNode, markedNode);
          }
          else {
              var textNode_1 = createTextNode(text);
              parent === null || parent === void 0 ? void 0 : parent.insertBefore(textNode_1, textNode_1);
          }
      }
  }
  function createTextNode(text) {
      return document.createTextNode(text);
  }
  function createMarkedNode(text) {
      var markElement = document.createElement('mark');
      var textNode = createTextNode(text);
      markElement.appendChild(textNode);
      return markElement;
  }
  var DOMHighlighter = {
      highlightHTMLElement: highlightHTMLElement,
      highlightHTMLElementWithTokens: highlightHTMLElementWithTokens
  };

  var createHighlightList$1 = highlighter.createHighlightList;
  var createHighlightListFromTokens$1 = highlighter.createHighlightListFromTokens;
  var tokenize$1 = tokenizer.tokenize;
  var highlightElement = DOMHighlighter.highlightHTMLElement;
  var highlightElementWithTokens = DOMHighlighter.highlightHTMLElementWithTokens;
  var fluo = {
      createHighlightList: highlighter.createHighlightList,
      createHighlightListFromTokens: highlighter.createHighlightListFromTokens,
      tokenize: tokenizer.tokenize,
      stringContainsAnyOfQuery: highlighter.stringContainsAnyOfQuery,
      stringContainsAnyOfTokens: highlighter.stringContainsAnyTokens,
      highlightElement: DOMHighlighter.highlightHTMLElement,
      highlightElementWithTokens: DOMHighlighter.highlightHTMLElementWithTokens
  };

  exports.createHighlightList = createHighlightList$1;
  exports.createHighlightListFromTokens = createHighlightListFromTokens$1;
  exports.default = fluo;
  exports.highlightElement = highlightElement;
  exports.highlightElementWithTokens = highlightElementWithTokens;
  exports.tokenize = tokenize$1;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=fluo.js.map
